@@ -1,44 +1,54 @@
 import dataclasses
+import json
 import os
 import pathlib
 import sys
 import uuid
-from datetime import datetime, timezone
+from collections.abc import Hashable
+from datetime import date, datetime, timezone
 from enum import IntEnum
 from pathlib import Path
-from typing import Any, Callable, Dict, Generic, Hashable, List, Optional, Set, Tuple, Type, TypeVar, Union
+from typing import Annotated, Any, Callable, Generic, Literal, Optional, TypeVar, Union
+from unittest import mock
 
 import pytest
 from annotated_types import MinLen
 from pydantic import (
     AliasChoices,
+    AliasGenerator,
     AliasPath,
     BaseModel,
     Discriminator,
     Field,
     HttpUrl,
     Json,
+    PostgresDsn,
     RootModel,
+    Secret,
     SecretStr,
     Tag,
     ValidationError,
+    field_validator,
 )
 from pydantic import (
     dataclasses as pydantic_dataclasses,
 )
 from pydantic.fields import FieldInfo
-from typing_extensions import Annotated, Literal, override
+from typing_extensions import override
 
 from pydantic_settings import (
     BaseSettings,
     DotEnvSettingsSource,
     EnvSettingsSource,
+    ForceDecode,
     InitSettingsSource,
+    NoDecode,
     PydanticBaseSettingsSource,
     SecretsSettingsSource,
     SettingsConfigDict,
+    SettingsError,
 )
-from pydantic_settings.sources import DefaultSettingsSource, SettingsError
+from pydantic_settings.sources import DefaultSettingsSource
 
 try:
     import dotenv
@@ -68,6 +78,12 @@ class SettingWithPopulateByName(BaseSettings):
     model_config = SettingsConfigDict(populate_by_name=True)
 
 
+@pytest.fixture(autouse=True)
+def clean_env():
+    with mock.patch.dict(os.environ, clear=True):
+        yield
+
+
 def test_sub_env(env):
     env.set('apple', 'hello')
     s = SimpleSettings()
@@ -186,7 +202,7 @@ def test_populate_by_name_with_alias_path_when_using_name(env):
         pytest.param({'pomme': 'pomme-chosen', 'manzano': 'manzano-chosen'}, 'pomme-chosen', id='pomme-priority'),
     ],
 )
-def test_populate_by_name_with_alias_choices_when_using_alias(env, env_vars: Dict[str, str], expected_value: str):
+def test_populate_by_name_with_alias_choices_when_using_alias(env, env_vars: dict[str, str], expected_value: str):
     for k, v in env_vars.items():
         env.set(k, v)
 
@@ -266,7 +282,7 @@ def test_nested_env_with_basemodel(env):
 
 def test_merge_dict(env):
     class Settings(BaseSettings):
-        top: Dict[str, str]
+        top: dict[str, str]
 
     with pytest.raises(ValidationError):
         Settings()
@@ -320,7 +336,7 @@ def test_nested_env_delimiter(env):
 
 def test_nested_env_optional_json(env):
     class Child(BaseModel):
-        num_list: Optional[List[int]] = None
+        num_list: Optional[list[int]] = None
 
     class Cfg(BaseSettings, env_nested_delimiter='__'):
         child: Optional[Child] = None
@@ -384,13 +400,47 @@ def test_nested_env_delimiter_aliases(env):
     assert Cfg().model_dump() == {'sub_model': {'v1': '-1-', 'v2': '-2-'}}
 
 
+@pytest.mark.parametrize('env_prefix', [None, 'prefix_', 'prefix__'])
+def test_nested_env_max_split(env, env_prefix):
+    class Person(BaseModel):
+        sex: Literal['M', 'F']
+        first_name: str
+        date_of_birth: date
+
+    class Cfg(BaseSettings):
+        caregiver: Person
+        significant_other: Optional[Person] = None
+        next_of_kin: Optional[Person] = None
+
+        model_config = SettingsConfigDict(env_nested_delimiter='_', env_nested_max_split=1)
+        if env_prefix is not None:
+            model_config['env_prefix'] = env_prefix
+
+    env_prefix = env_prefix or ''
+    env.set(env_prefix + 'caregiver_sex', 'M')
+    env.set(env_prefix + 'caregiver_first_name', 'Joe')
+    env.set(env_prefix + 'caregiver_date_of_birth', '1975-09-12')
+    env.set(env_prefix + 'significant_other_sex', 'F')
+    env.set(env_prefix + 'significant_other_first_name', 'Jill')
+    env.set(env_prefix + 'significant_other_date_of_birth', '1998-04-19')
+    env.set(env_prefix + 'next_of_kin_sex', 'M')
+    env.set(env_prefix + 'next_of_kin_first_name', 'Jack')
+    env.set(env_prefix + 'next_of_kin_date_of_birth', '1999-04-19')
+
+    assert Cfg().model_dump() == {
+        'caregiver': {'sex': 'M', 'first_name': 'Joe', 'date_of_birth': date(1975, 9, 12)},
+        'significant_other': {'sex': 'F', 'first_name': 'Jill', 'date_of_birth': date(1998, 4, 19)},
+        'next_of_kin': {'sex': 'M', 'first_name': 'Jack', 'date_of_birth': date(1999, 4, 19)},
+    }
+
+
 class DateModel(BaseModel):
     pips: bool = False
 
 
 class ComplexSettings(BaseSettings):
-    apples: List[str] = []
-    bananas: Set[int] = set()
+    apples: list[str] = []
+    bananas: set[int] = set()
     carrots: dict = {}
     date: DateModel = DateModel()
 
@@ -404,7 +454,7 @@ def test_list(env):
 
 def test_annotated_list(env):
     class AnnotatedComplexSettings(BaseSettings):
-        apples: Annotated[List[str], MinLen(2)] = []
+        apples: Annotated[list[str], MinLen(2)] = []
 
     env.set('apples', '["russet", "granny smith"]')
     s = AnnotatedComplexSettings()
@@ -614,6 +664,37 @@ def test_init_kwargs_nested_model_default_partial_update(env):
     assert s.model_dump() == s_final
 
 
+def test_alias_resolution_init_source(env):
+    class Example(BaseSettings):
+        model_config = SettingsConfigDict(env_prefix='PREFIX')
+
+        name: str
+        last_name: str = Field(validation_alias=AliasChoices('PREFIX_LAST_NAME', 'PREFIX_SURNAME'))
+
+    env.set('PREFIX_SURNAME', 'smith')
+    assert Example(name='john', PREFIX_SURNAME='doe').model_dump() == {'name': 'john', 'last_name': 'doe'}
+
+
+def test_alias_nested_model_default_partial_update():
+    class SubModel(BaseModel):
+        v1: str = 'default'
+        v2: bytes = b'hello'
+        v3: int
+
+    class Settings(BaseSettings):
+        model_config = SettingsConfigDict(
+            nested_model_default_partial_update=True, alias_generator=AliasGenerator(lambda s: s.replace('_', '-'))
+        )
+
+        v0: str = 'ok'
+        sub_model: SubModel = SubModel(v1='top default', v3=33)
+
+    assert Settings(**{'sub-model': {'v1': 'cli'}}).model_dump() == {
+        'v0': 'ok',
+        'sub_model': {'v1': 'cli', 'v2': b'hello', 'v3': 33},
+    }
+
+
 def test_env_str(env):
     class Settings(BaseSettings):
         apple: str = Field(None, validation_alias='BOOM')
@@ -790,6 +871,25 @@ def test_case_sensitive(monkeypatch):
     ]
 
 
+@pytest.mark.parametrize('env_nested_delimiter', [None, ''])
+def test_case_sensitive_no_nested_delimiter(monkeypatch, env_nested_delimiter):
+    class Subsettings(BaseSettings):
+        foo: str
+
+    class Settings(BaseSettings):
+        subsettings: Subsettings
+
+        model_config = SettingsConfigDict(case_sensitive=True, env_nested_delimiter=env_nested_delimiter)
+
+    # Need to patch os.environ to get build to work on Windows, where os.environ is case insensitive
+    monkeypatch.setattr(os, 'environ', value={'subsettingsNonefoo': '1'})
+    with pytest.raises(ValidationError) as exc_info:
+        Settings()
+    assert exc_info.value.errors(include_url=False) == [
+        {'type': 'missing', 'loc': ('subsettings',), 'msg': 'Field required', 'input': {}}
+    ]
+
+
 def test_nested_dataclass(env):
     @pydantic_dataclasses.dataclass
     class DeepNestedDataclass:
@@ -813,7 +913,7 @@ def test_nested_dataclass(env):
     assert s.n.bar == 'bar value'
 
 
-def test_nested_vanila_dataclass(env):
+def test_nested_vanilla_dataclass(env):
     @dataclasses.dataclass
     class MyDataclass:
         value: str
@@ -839,12 +939,12 @@ def test_env_takes_precedence(env):
         @classmethod
         def settings_customise_sources(
             cls,
-            settings_cls: Type[BaseSettings],
+            settings_cls: type[BaseSettings],
             init_settings: PydanticBaseSettingsSource,
             env_settings: PydanticBaseSettingsSource,
             dotenv_settings: PydanticBaseSettingsSource,
             file_secret_settings: PydanticBaseSettingsSource,
-        ) -> Tuple[PydanticBaseSettingsSource, ...]:
+        ) -> tuple[PydanticBaseSettingsSource, ...]:
             return env_settings, init_settings
 
     env.set('BAR', 'env setting')
@@ -859,7 +959,7 @@ def test_config_file_settings_nornir(env):
     See https://github.com/pydantic/pydantic/pull/341#issuecomment-450378771
     """
 
-    def nornir_settings_source() -> Dict[str, Any]:
+    def nornir_settings_source() -> dict[str, Any]:
         return {'param_a': 'config a', 'param_b': 'config b', 'param_c': 'config c'}
 
     class Settings(BaseSettings):
@@ -870,12 +970,12 @@ def test_config_file_settings_nornir(env):
         @classmethod
         def settings_customise_sources(
             cls,
-            settings_cls: Type[BaseSettings],
+            settings_cls: type[BaseSettings],
             init_settings: PydanticBaseSettingsSource,
             env_settings: PydanticBaseSettingsSource,
             dotenv_settings: PydanticBaseSettingsSource,
             file_secret_settings: PydanticBaseSettingsSource,
-        ) -> Tuple[PydanticBaseSettingsSource, ...]:
+        ) -> tuple[PydanticBaseSettingsSource, ...]:
             return env_settings, init_settings, nornir_settings_source
 
     env.set('PARAM_C', 'env setting c')
@@ -1109,9 +1209,13 @@ def test_env_file_config_custom_encoding(tmp_path):
 
 
 @pytest.fixture
-def home_tmp():
+def home_tmp(tmp_path, env):
+    env.set('HOME', str(tmp_path))
+    env.set('USERPROFILE', str(tmp_path))
+    env.set('HOMEPATH', str(tmp_path))
+
     tmp_filename = f'{uuid.uuid4()}.env'
-    home_tmp_path = Path.home() / tmp_filename
+    home_tmp_path = tmp_path / tmp_filename
     yield home_tmp_path, tmp_filename
     home_tmp_path.unlink()
 
@@ -1332,12 +1436,12 @@ def test_read_dotenv_vars_when_env_file_is_none():
 def test_dotenvsource_override(env):
     class StdinDotEnvSettingsSource(DotEnvSettingsSource):
         @override
-        def _read_env_file(self, file_path: Path) -> Dict[str, str]:
+        def _read_env_file(self, file_path: Path) -> dict[str, str]:
             assert str(file_path) == '-'
             return {'foo': 'stdin_foo', 'bar': 'stdin_bar'}
 
         @override
-        def _read_env_files(self) -> Dict[str, str]:
+        def _read_env_files(self) -> dict[str, str]:
             return self._read_env_file(Path('-'))
 
     source = StdinDotEnvSettingsSource(BaseSettings())
@@ -1495,7 +1599,7 @@ def test_secrets_path_json(tmp_path):
     p.write_text('{"a": "b"}')
 
     class Settings(BaseSettings):
-        foo: Dict[str, str]
+        foo: dict[str, str]
 
         model_config = SettingsConfigDict(secrets_dir=tmp_path)
 
@@ -1522,7 +1626,7 @@ def test_secrets_path_invalid_json(tmp_path):
     p.write_text('{"a": "b"')
 
     class Settings(BaseSettings):
-        foo: Dict[str, str]
+        foo: dict[str, str]
 
         model_config = SettingsConfigDict(secrets_dir=tmp_path)
 
@@ -1533,7 +1637,7 @@ def test_secrets_path_invalid_json(tmp_path):
 def test_secrets_missing(tmp_path):
     class Settings(BaseSettings):
         foo: str
-        bar: List[str]
+        bar: list[str]
 
         model_config = SettingsConfigDict(secrets_dir=tmp_path)
 
@@ -1628,7 +1732,9 @@ def test_secrets_file_is_a_directory(tmp_path):
 
         model_config = SettingsConfigDict(secrets_dir=tmp_path)
 
-    with pytest.warns(UserWarning, match=f'attempted to load secret file "{tmp_path}/foo" but found a directory inste'):
+    with pytest.warns(
+        UserWarning, match=f'attempted to load secret file "{tmp_path}/foo" but found a directory instead'
+    ):
         Settings()
 
 
@@ -1684,10 +1790,10 @@ def test_secrets_dotenv_precedence(tmp_path):
 
 
 def test_external_settings_sources_precedence(env):
-    def external_source_0() -> Dict[str, str]:
+    def external_source_0() -> dict[str, str]:
         return {'apple': 'value 0', 'banana': 'value 2'}
 
-    def external_source_1() -> Dict[str, str]:
+    def external_source_1() -> dict[str, str]:
         return {'apple': 'value 1', 'raspberry': 'value 3'}
 
     class Settings(BaseSettings):
@@ -1698,12 +1804,12 @@ def test_external_settings_sources_precedence(env):
         @classmethod
         def settings_customise_sources(
             cls,
-            settings_cls: Type[BaseSettings],
+            settings_cls: type[BaseSettings],
             init_settings: PydanticBaseSettingsSource,
             env_settings: PydanticBaseSettingsSource,
             dotenv_settings: PydanticBaseSettingsSource,
             file_secret_settings: PydanticBaseSettingsSource,
-        ) -> Tuple[PydanticBaseSettingsSource, ...]:
+        ) -> tuple[PydanticBaseSettingsSource, ...]:
             return (
                 init_settings,
                 env_settings,
@@ -1721,7 +1827,7 @@ def test_external_settings_sources_filter_env_vars():
     vault_storage = {'user:password': {'apple': 'value 0', 'banana': 'value 2'}}
 
     class VaultSettingsSource(PydanticBaseSettingsSource):
-        def __init__(self, settings_cls: Type[BaseSettings], user: str, password: str):
+        def __init__(self, settings_cls: type[BaseSettings], user: str, password: str):
             self.user = user
             self.password = password
             super().__init__(settings_cls)
@@ -1729,7 +1835,7 @@ def test_external_settings_sources_filter_env_vars():
         def get_field_value(self, field: FieldInfo, field_name: str) -> Any:
             pass
 
-        def __call__(self) -> Dict[str, str]:
+        def __call__(self) -> dict[str, str]:
             vault_vars = vault_storage[f'{self.user}:{self.password}']
             return {
                 field_name: vault_vars[field_name]
@@ -1744,12 +1850,12 @@ def test_external_settings_sources_filter_env_vars():
         @classmethod
         def settings_customise_sources(
             cls,
-            settings_cls: Type[BaseSettings],
+            settings_cls: type[BaseSettings],
             init_settings: PydanticBaseSettingsSource,
             env_settings: PydanticBaseSettingsSource,
             dotenv_settings: PydanticBaseSettingsSource,
             file_secret_settings: PydanticBaseSettingsSource,
-        ) -> Tuple[PydanticBaseSettingsSource, ...]:
+        ) -> tuple[PydanticBaseSettingsSource, ...]:
             return (
                 init_settings,
                 env_settings,
@@ -1788,8 +1894,7 @@ def test_builtins_settings_source_repr():
         == "EnvSettingsSource(env_nested_delimiter='__', env_prefix_len=0)"
     )
     assert repr(DotEnvSettingsSource(BaseSettings, env_file='.env', env_file_encoding='utf-8')) == (
-        "DotEnvSettingsSource(env_file='.env', env_file_encoding='utf-8', "
-        'env_nested_delimiter=None, env_prefix_len=0)'
+        "DotEnvSettingsSource(env_file='.env', env_file_encoding='utf-8', env_nested_delimiter=None, env_prefix_len=0)"
     )
     assert (
         repr(SecretsSettingsSource(BaseSettings, secrets_dir='/secrets'))
@@ -1797,7 +1902,7 @@ def test_builtins_settings_source_repr():
     )
 
 
-def _parse_custom_dict(value: str) -> Callable[[str], Dict[int, str]]:
+def _parse_custom_dict(value: str) -> Callable[[str], dict[int, str]]:
     """A custom parsing function passed into env parsing test."""
     res = {}
     for part in value.split(','):
@@ -1816,17 +1921,17 @@ class CustomEnvSettingsSource(EnvSettingsSource):
 
 def test_env_setting_source_custom_env_parse(env):
     class Settings(BaseSettings):
-        top: Dict[int, str]
+        top: dict[int, str]
 
         @classmethod
         def settings_customise_sources(
             cls,
-            settings_cls: Type[BaseSettings],
+            settings_cls: type[BaseSettings],
             init_settings: PydanticBaseSettingsSource,
             env_settings: PydanticBaseSettingsSource,
             dotenv_settings: PydanticBaseSettingsSource,
             file_secret_settings: PydanticBaseSettingsSource,
-        ) -> Tuple[PydanticBaseSettingsSource, ...]:
+        ) -> tuple[PydanticBaseSettingsSource, ...]:
             return (CustomEnvSettingsSource(settings_cls),)
 
     with pytest.raises(ValidationError):
@@ -1844,17 +1949,17 @@ class BadCustomEnvSettingsSource(EnvSettingsSource):
 
 def test_env_settings_source_custom_env_parse_is_bad(env):
     class Settings(BaseSettings):
-        top: Dict[int, str]
+        top: dict[int, str]
 
         @classmethod
         def settings_customise_sources(
             cls,
-            settings_cls: Type[BaseSettings],
+            settings_cls: type[BaseSettings],
             init_settings: PydanticBaseSettingsSource,
             env_settings: PydanticBaseSettingsSource,
             dotenv_settings: PydanticBaseSettingsSource,
             file_secret_settings: PydanticBaseSettingsSource,
-        ) -> Tuple[PydanticBaseSettingsSource, ...]:
+        ) -> tuple[PydanticBaseSettingsSource, ...]:
             return (BadCustomEnvSettingsSource(settings_cls),)
 
     env.set('top', '1=apple,2=banana')
@@ -1877,18 +1982,18 @@ def test_secret_settings_source_custom_env_parse(tmp_path):
     p.write_text('1=apple,2=banana')
 
     class Settings(BaseSettings):
-        top: Dict[int, str]
+        top: dict[int, str]
 
         model_config = SettingsConfigDict(secrets_dir=tmp_path)
 
         def settings_customise_sources(
             cls,
-            settings_cls: Type[BaseSettings],
+            settings_cls: type[BaseSettings],
             init_settings: PydanticBaseSettingsSource,
             env_settings: PydanticBaseSettingsSource,
             dotenv_settings: PydanticBaseSettingsSource,
             file_secret_settings: PydanticBaseSettingsSource,
-        ) -> Tuple[PydanticBaseSettingsSource, ...]:
+        ) -> tuple[PydanticBaseSettingsSource, ...]:
             return (CustomSecretsSettingsSource(settings_cls, tmp_path),)
 
     s = Settings()
@@ -1902,17 +2007,17 @@ class BadCustomSettingsSource(EnvSettingsSource):
 
 def test_custom_source_get_field_value_error(env):
     class Settings(BaseSettings):
-        top: Dict[int, str]
+        top: dict[int, str]
 
         @classmethod
         def settings_customise_sources(
             cls,
-            settings_cls: Type[BaseSettings],
+            settings_cls: type[BaseSettings],
             init_settings: PydanticBaseSettingsSource,
             env_settings: PydanticBaseSettingsSource,
             dotenv_settings: PydanticBaseSettingsSource,
             file_secret_settings: PydanticBaseSettingsSource,
-        ) -> Tuple[PydanticBaseSettingsSource, ...]:
+        ) -> tuple[PydanticBaseSettingsSource, ...]:
             return (BadCustomSettingsSource(settings_cls),)
 
     with pytest.raises(
@@ -1923,10 +2028,10 @@ def test_custom_source_get_field_value_error(env):
 
 def test_nested_env_complex_values(env):
     class SubSubModel(BaseSettings):
-        dvals: Dict
+        dvals: dict
 
     class SubModel(BaseSettings):
-        vals: List[str]
+        vals: list[str]
         sub_sub_model: SubSubModel
 
     class Cfg(BaseSettings):
@@ -1948,7 +2053,7 @@ def test_nested_env_complex_values(env):
 
 def test_nested_env_nonexisting_field(env):
     class SubModel(BaseSettings):
-        vals: List[str]
+        vals: list[str]
 
     class Cfg(BaseSettings):
         sub_model: SubModel
@@ -1962,7 +2067,7 @@ def test_nested_env_nonexisting_field(env):
 
 def test_nested_env_nonexisting_field_deep(env):
     class SubModel(BaseSettings):
-        vals: List[str]
+        vals: list[str]
 
     class Cfg(BaseSettings):
         sub_model: SubModel
@@ -1976,7 +2081,7 @@ def test_nested_env_nonexisting_field_deep(env):
 
 def test_nested_env_union_complex_values(env):
     class SubModel(BaseSettings):
-        vals: Union[List[str], Dict[str, str]]
+        vals: Union[list[str], dict[str, str]]
 
     class Cfg(BaseSettings):
         sub_model: SubModel
@@ -2168,26 +2273,28 @@ def test_nested_bytes_field(env):
 def test_protected_namespace_defaults():
     # pydantic default
     with pytest.warns(
-        UserWarning, match='Field "model_prefixed_field" in Model has conflict with protected namespace "model_"'
+        UserWarning,
+        match='Field "model_dump_prefixed_field" in Model has conflict with protected namespace "model_dump"',
     ):
 
         class Model(BaseSettings):
-            model_prefixed_field: str
+            model_dump_prefixed_field: str
 
     # pydantic-settings default
-    with pytest.raises(
-        UserWarning, match='Field "settings_prefixed_field" in Model1 has conflict with protected namespace "settings_"'
+    with pytest.warns(
+        UserWarning,
+        match='Field "settings_customise_sources_prefixed_field" in Model1 has conflict with protected namespace "settings_customise_sources"',
     ):
 
         class Model1(BaseSettings):
-            settings_prefixed_field: str
+            settings_customise_sources_prefixed_field: str
 
     with pytest.raises(
         NameError,
         match=(
             'Field "settings_customise_sources" conflicts with member <bound method '
             "BaseSettings.settings_customise_sources of <class 'pydantic_settings.main.BaseSettings'>> "
-            'of protected namespace "settings_".'
+            'of protected namespace "settings_customise_sources".'
         ),
     ):
 
@@ -2241,13 +2348,18 @@ def test_env_json_field(env):
 
 
 def test_env_parse_enums(env):
-    class Settings(BaseSettings):
+    class NestedEnum(BaseModel):
+        fruit: FruitsEnum
+
+    class Settings(BaseSettings, env_nested_delimiter='__'):
         fruit: FruitsEnum
         union_fruit: Optional[Union[int, FruitsEnum]] = None
+        nested: NestedEnum
 
     with pytest.raises(ValidationError) as exc_info:
         env.set('FRUIT', 'kiwi')
         env.set('UNION_FRUIT', 'kiwi')
+        env.set('NESTED__FRUIT', 'kiwi')
         s = Settings()
     assert exc_info.value.errors(include_url=False) == [
         {
@@ -2278,22 +2390,43 @@ def test_env_parse_enums(env):
             'msg': 'Input should be 0, 1 or 2',
             'type': 'enum',
         },
+        {
+            'ctx': {
+                'expected': '0, 1 or 2',
+            },
+            'input': 'kiwi',
+            'loc': (
+                'nested',
+                'fruit',
+            ),
+            'msg': 'Input should be 0, 1 or 2',
+            'type': 'enum',
+        },
     ]
 
     env.set('FRUIT', str(FruitsEnum.lime.value))
     env.set('UNION_FRUIT', str(FruitsEnum.lime.value))
+    env.set('NESTED__FRUIT', str(FruitsEnum.lime.value))
     s = Settings()
     assert s.fruit == FruitsEnum.lime
+    assert s.union_fruit == FruitsEnum.lime
+    assert s.nested.fruit == FruitsEnum.lime
 
     env.set('FRUIT', 'kiwi')
     env.set('UNION_FRUIT', 'kiwi')
+    env.set('NESTED__FRUIT', 'kiwi')
     s = Settings(_env_parse_enums=True)
     assert s.fruit == FruitsEnum.kiwi
+    assert s.union_fruit == FruitsEnum.kiwi
+    assert s.nested.fruit == FruitsEnum.kiwi
 
     env.set('FRUIT', str(FruitsEnum.lime.value))
     env.set('UNION_FRUIT', str(FruitsEnum.lime.value))
+    env.set('NESTED__FRUIT', str(FruitsEnum.lime.value))
     s = Settings(_env_parse_enums=True)
     assert s.fruit == FruitsEnum.lime
+    assert s.union_fruit == FruitsEnum.lime
+    assert s.nested.fruit == FruitsEnum.lime
 
 
 def test_env_parse_none_str(env):
@@ -2358,7 +2491,7 @@ def test_env_parse_none_str(env):
 
 def test_env_json_field_dict(env):
     class Settings(BaseSettings):
-        x: Json[Dict[str, int]]
+        x: Json[dict[str, int]]
 
     env.set('x', '{"foo": 1}')
 
@@ -2413,9 +2546,9 @@ def test_model_config_through_class_kwargs(env):
 def test_root_model_as_field(env):
     class Foo(BaseModel):
         x: int
-        y: Dict[str, int]
+        y: dict[str, int]
 
-    FooRoot = RootModel[List[Foo]]
+    FooRoot = RootModel[list[Foo]]
 
     class Settings(BaseSettings):
         z: FooRoot
@@ -2479,7 +2612,7 @@ def test_dotenv_optional_json_field(tmp_path):
     class Settings(BaseSettings):
         model_config = SettingsConfigDict(env_file=p)
 
-        data: Optional[Json[Dict[str, str]]] = Field(default=None)
+        data: Optional[Json[dict[str, str]]] = Field(default=None)
 
     s = Settings()
     assert s.data == {'foo': 'bar'}
@@ -2556,11 +2689,11 @@ def test_nested_field_with_alias_init_source():
 
 def test_nested_models_as_dict_value(env):
     class NestedSettings(BaseModel):
-        foo: Dict[str, int]
+        foo: dict[str, int]
 
     class Settings(BaseSettings):
         nested: NestedSettings
-        sub_dict: Dict[str, NestedSettings]
+        sub_dict: dict[str, NestedSettings]
 
         model_config = SettingsConfigDict(env_nested_delimiter='__')
 
@@ -2572,7 +2705,7 @@ def test_nested_models_as_dict_value(env):
 
 def test_env_nested_dict_value(env):
     class Settings(BaseSettings):
-        nested: Dict[str, Dict[str, Dict[str, str]]]
+        nested: dict[str, dict[str, dict[str, str]]]
 
         model_config = SettingsConfigDict(env_nested_delimiter='__')
 
@@ -2621,7 +2754,7 @@ def test_case_insensitive_nested_optional(env):
 
 def test_case_insensitive_nested_list(env):
     class NestedSettings(BaseModel):
-        FOO: List[str]
+        FOO: list[str]
 
     class Settings(BaseSettings):
         model_config = SettingsConfigDict(env_nested_delimiter='__', case_sensitive=False)
@@ -2638,7 +2771,7 @@ def test_settings_source_current_state(env):
         def get_field_value(self, field: FieldInfo, field_name: str) -> Any:
             pass
 
-        def __call__(self) -> Dict[str, Any]:
+        def __call__(self) -> dict[str, Any]:
             current_state = self.current_state
             if current_state.get('one') == '1':
                 return {'two': '1'}
@@ -2652,12 +2785,12 @@ def test_settings_source_current_state(env):
         @classmethod
         def settings_customise_sources(
             cls,
-            settings_cls: Type[BaseSettings],
+            settings_cls: type[BaseSettings],
             init_settings: PydanticBaseSettingsSource,
             env_settings: PydanticBaseSettingsSource,
             dotenv_settings: PydanticBaseSettingsSource,
             file_secret_settings: PydanticBaseSettingsSource,
-        ) -> Tuple[PydanticBaseSettingsSource, ...]:
+        ) -> tuple[PydanticBaseSettingsSource, ...]:
             return (env_settings, SettingsSource(settings_cls))
 
     env.set('one', '1')
@@ -2670,7 +2803,7 @@ def test_settings_source_settings_sources_data(env):
         def get_field_value(self, field: FieldInfo, field_name: str) -> Any:
             pass
 
-        def __call__(self) -> Dict[str, Any]:
+        def __call__(self) -> dict[str, Any]:
             settings_sources_data = self.settings_sources_data
             if settings_sources_data == {
                 'InitSettingsSource': {'one': True, 'two': True},
@@ -2693,12 +2826,12 @@ def test_settings_source_settings_sources_data(env):
         @classmethod
         def settings_customise_sources(
             cls,
-            settings_cls: Type[BaseSettings],
+            settings_cls: type[BaseSettings],
             init_settings: PydanticBaseSettingsSource,
             env_settings: PydanticBaseSettingsSource,
             dotenv_settings: PydanticBaseSettingsSource,
             file_secret_settings: PydanticBaseSettingsSource,
-        ) -> Tuple[PydanticBaseSettingsSource, ...]:
+        ) -> tuple[PydanticBaseSettingsSource, ...]:
             return (env_settings, init_settings, function_settings_source, SettingsSource(settings_cls))
 
     env.set('one', '1')
@@ -2720,7 +2853,6 @@ def test_dotenv_extra_allow_similar_fields(tmp_path):
     assert s.model_dump() == {'POSTGRES_USER': 'postgres', 'postgres_name': 'name', 'postgres_user_2': 'postgres2'}
 
 
-@pytest.mark.skipif(sys.version_info < (3, 9), reason='requires python 3.9 or higher')
 def test_annotation_is_complex_root_model_check():
     """Test for https://github.com/pydantic/pydantic-settings/issues/390"""
 
@@ -2732,7 +2864,7 @@ def test_annotation_is_complex_root_model_check():
 
 def test_nested_model_field_with_alias(env):
     class NestedSettings(BaseModel):
-        foo: List[str] = Field(alias='fooalias')
+        foo: list[str] = Field(alias='fooalias')
 
     class Settings(BaseSettings):
         model_config = SettingsConfigDict(env_nested_delimiter='__')
@@ -2747,7 +2879,7 @@ def test_nested_model_field_with_alias(env):
 
 def test_nested_model_field_with_alias_case_sensitive(monkeypatch):
     class NestedSettings(BaseModel):
-        foo: List[str] = Field(alias='fooAlias')
+        foo: list[str] = Field(alias='fooAlias')
 
     class Settings(BaseSettings):
         model_config = SettingsConfigDict(env_nested_delimiter='__', case_sensitive=True)
@@ -2774,7 +2906,7 @@ def test_nested_model_field_with_alias_case_sensitive(monkeypatch):
 
 def test_nested_model_field_with_alias_choices(env):
     class NestedSettings(BaseModel):
-        foo: List[str] = Field(alias=AliasChoices('fooalias', 'foo-alias'))
+        foo: list[str] = Field(alias=AliasChoices('fooalias', 'foo-alias'))
 
     class Settings(BaseSettings):
         model_config = SettingsConfigDict(env_nested_delimiter='__')
@@ -2824,3 +2956,82 @@ def test_dotenv_env_prefix_env_without_prefix(tmp_path):
 
     s = Settings()
     assert s.model_dump() == {'foo': 'test-foo'}
+
+
+def test_parsing_secret_field(env):
+    class Settings(BaseSettings):
+        foo: Secret[int]
+        bar: Secret[PostgresDsn]
+
+    env.set('foo', '123')
+    env.set('bar', 'postgres://user:password@localhost/dbname')
+
+    s = Settings()
+    assert s.foo.get_secret_value() == 123
+    assert s.bar.get_secret_value() == PostgresDsn('postgres://user:password@localhost/dbname')
+
+
+def test_field_annotated_no_decode(env):
+    class Settings(BaseSettings):
+        a: list[str]  # this field will be decoded because of default `enable_decoding=True`
+        b: Annotated[list[str], NoDecode]
+
+        # decode the value here. the field value won't be decoded because of NoDecode
+        @field_validator('b', mode='before')
+        @classmethod
+        def decode_b(cls, v: str) -> list[str]:
+            return json.loads(v)
+
+    env.set('a', '["one", "two"]')
+    env.set('b', '["1", "2"]')
+
+    s = Settings()
+    assert s.model_dump() == {'a': ['one', 'two'], 'b': ['1', '2']}
+
+
+def test_field_annotated_no_decode_and_disable_decoding(env):
+    class Settings(BaseSettings):
+        model_config = SettingsConfigDict(enable_decoding=False)
+
+        a: Annotated[list[str], NoDecode]
+
+        # decode the value here. the field value won't be decoded because of NoDecode
+        @field_validator('a', mode='before')
+        @classmethod
+        def decode_b(cls, v: str) -> list[str]:
+            return json.loads(v)
+
+    env.set('a', '["one", "two"]')
+
+    s = Settings()
+    assert s.model_dump() == {'a': ['one', 'two']}
+
+
+def test_field_annotated_disable_decoding(env):
+    class Settings(BaseSettings):
+        model_config = SettingsConfigDict(enable_decoding=False)
+
+        a: list[str]
+
+        # decode the value here. the field value won't be decoded because of `enable_decoding=False`
+        @field_validator('a', mode='before')
+        @classmethod
+        def decode_b(cls, v: str) -> list[str]:
+            return json.loads(v)
+
+    env.set('a', '["one", "two"]')
+
+    s = Settings()
+    assert s.model_dump() == {'a': ['one', 'two']}
+
+
+def test_field_annotated_force_decode_disable_decoding(env):
+    class Settings(BaseSettings):
+        model_config = SettingsConfigDict(enable_decoding=False)
+
+        a: Annotated[list[str], ForceDecode]
+
+    env.set('a', '["one", "two"]')
+
+    s = Settings()
+    assert s.model_dump() == {'a': ['one', 'two']}
